(function($) {

    let MQ = MathQuill.getInterface(2);

    /**
     * CheckerTool class
     * @class CheckerTool
     * @constructor
     * @param {Object} editor   The sdeditor object
     */
    let CheckerTool = function(editor) {
        this.editor = editor;
        this.uilang = this.editor.settings.uilang;
    };

    // Class variables and functions
    //-------------------------------------
    CheckerTool.styles = '[data-focusonelem="sdterm"] .sdeditor-Checker-compute { display: block !important; }';

    /**
     * Add necessary styles (if not already added).
     */
    CheckerTool.addStyles = function(uilang) {
        if ($('head style#checker-style').length === 0) {
            $('head').append('<style id="checker-style" type="text/css">'+CheckerTool.styles+'</style>')
        };
    }

    CheckerTool.state = {
        confRegistered: false
    };

    /**
     * Register config options for config tool
     * @param {String} uilang   The language for user interface.
     */
    CheckerTool.registerConfigs = function(uilang) {
        if (!CheckerTool.state.confRegistered) {
            let place = $('.virumcourse-wrapper');
            if (place.length > 0) {
                let listener = $('<div class="checkertool-listener configdialog-listener" data-appname="checker"></div>');
                $('body').append(listener);
                $('body').off('setconfigs.checkertool').on('setconfigs.checkertool', '.checkertool-listener', function(event, data) {
                    event.stopPropagation();
                    CheckerTool.setConfigs(data);
                }).off('reply_getconfigs.checkertool').on('reply_getconfigs.checkertool', '.checkertool-listener', function(event, data) {
                    event.stopPropagation();
                    CheckerTool.setConfigs(data);
                }).off('reply_getconnections.checkertool').on('reply_getconnections.checkertool', '.checkertool-listener', function(event, data) {
                    event.stopPropagation();
                    CheckerTool.setConnections(data);
                }).off('reply_getuserinfo.checkertool').on('reply_getuserinfo.checkertool', '.checkertool-listener', function(event, data) {
                    event.stopPropagation();
                    CheckerTool.setUser(data);
                });
                let formdata = {apptype: 'checker', appid: 'checker', confform: CheckerTool.getConfigForm(uilang)};
                place.trigger('registerconfigdialog', formdata);
                CheckerTool.state.confRegistered = true;
            };
        };
    };

    /**
     * Register checker keybindings
     */
    CheckerTool.registerKeybindings = function(editor) {
        editor.place.off('keydown.checkertool').on('keydown.checkertool', '.mq-editable-field', function(event) {
            switch (event.which) {
                case 13:
                    if (event.ctrlKey) {
                        // Ctrl + Enter : Compute
                        event.stopPropagation();
                        CheckerTool.computeHandler(editor);
                        break;
                    }
                case 49:
                    if (event.ctrlKey) {
                        // Ctrl + ! : Check the derivation
                        event.stopPropagation();
                        let tool = new CheckerTool(editor);
                        tool.init();
                        tool.check();
                    }
            }
        });
    }

    /**
     * The user has requested the compute action. Check if in the right context (in a term), and if there is a
     * selection, wrap with \langle. Otherwser
     */
    CheckerTool.computeHandler = function(editor) {
        let tool = this;

        let current = editor.place.data('currentmq');
        if (current === null || typeof(current) === 'undefined') {
            return undefined;
        } else {
            let mq = MQ(current[0]);
            let loc = current.closest('td[data-loc]').attr('data-loc').split('_');
            if (typeof(loc) === 'undefined')
                return;
            let locType = loc[loc.length - 2];
            if (locType !== 'term')
                return;
            editor.place.data('currentmq').focusout();
            editor.place.data('currentmq').focusin();

            // Insert selection parens around the MathQuill selection (if exists).
            if (current.find('.mq-selection').length > 0) {
                mq.cmd('\\langle');
            }
            editor.place.data('currentmq').focusout();
            const targetLoc = loc.slice(0);
            let tool = new CheckerTool(editor);
            tool.init();
            tool.compute(targetLoc);
        }
    }

    CheckerTool.normalizeMQLatex = function(latex) {
       let mqSpan = MQ.StaticMath(document.createElement('span'));
       mqSpan.latex(latex);
       return mqSpan.latex();
    }

    /**
     * Get config form as JSON for sending to the config dialog.
      * @param {String} uilang   The language for user interface.
    */
    CheckerTool.getConfigForm = function(uilang){
        let form = {
            name: 'checker',
            title: this.localize('checker', uilang),
            icon: CheckerTool.prototype.icons.check,
            weight: 100,
            form: [
                {
                    name: 'decimalSeparator',
                    label: this.localize('decimalseparator', uilang),
                    type: 'select',
                    attribute: 'decimalSeparator',
                    defval: ',',
                    options: [
                        {
                            label: this.localize('comma', uilang) + ' (,)',
                            value: ','
                        },
                        {
                            label: this.localize('period', uilang) + ' (.)',
                            value: '.'
                        }
                    ]
                }
            ]
        };
        return form;
    };

    /**
     * Set configs that have been changed with config dialog.
     * @param {Object} configs   Config data
     */
    CheckerTool.setConfigs = function(configs) {
        CheckerTool.configs = $.extend(true, CheckerTool.configs, configs);
        $('.virumcourse-wrapper').children().eq(0).trigger('saveconfigs', {
            appname: 'checker',
            configs: this.configs,
            type: 'configsave'
        });
    };

    /**
     * Set connections that have been requested from the context.
     * @param {Object} connections   Connection urls
     */
    CheckerTool.setConnections = function(connections) {
        CheckerTool.connection = $.extend(true, CheckerTool.connection, connections);
    };

    /**
     * Set userinfo that have been requested from the context.
     * @param {Object} user   User info
     */
    CheckerTool.setUser = function(user) {
        if (user.applicationkey) {
            // Update only, if there is an applicationkey
            CheckerTool.user = $.extend(true, CheckerTool.user, user);
        };
    };

    /**
     * Localizations
     */
    CheckerTool.localize = function(term, uilang) {
        uilang = uilang || 'en';
        let langset = this.localizations[uilang] || this.localizations.en;
        return langset[term] || this.localizations.en[term] || term;
    };

    /**
     * Localization strings
     */
    CheckerTool.localizations = {
        en: {
            'checker': 'Checker',
            'decimalseparator': 'Decimal separator',
            'period': 'Period',
            'comma': 'Comma'
        },
        fi: {
            'checker': 'Tarkistin',
            'decimalseparator': 'Desimaalierotin',
            'period': 'Piste',
            'comma': 'Pilkku'
        },
        sv: {
            'checker': 'Checker',
            'decimalseparator': 'Decimaltecken',
            'period': 'Punkt',
            'comma': 'Komma'
        }
    }

    /**
     * Place for saved configs.
     */
    CheckerTool.configs = {

    }

    /**
     * Place for saved connections.
     */
    CheckerTool.connection = {
        homeURL: 'https://fourferries.com',
        serverURL: 'https://service4f.fourferries.com/service4f/app',
        checkerURL: 'wss://sdchecker.fourferries.com:9443/checker/prove',
        computeURL: 'wss://sdchecker.fourferries.com:9443/checker/compute'
    }

    /**
     * Place for user details
     */
    CheckerTool.user = {
        username: null,
        applicationkey: null
    }

    // Methods for class instances
    //--------------------------------------
    /**
     * Init the tool
     */
    CheckerTool.prototype.init = function() {
        this.getConfigs();
        this.getConnections();
        this.getUserinfo();
        this.checkData = $.extend(true, {},
            this.defaults,
            CheckerTool.configs,
            this.editor.getCheckData()
        );
        this.checkname = this.checkData.name;
        this.settings = {
            username: CheckerTool.user.username,
            appkey: CheckerTool.user.applicationkey,
            serverURL: CheckerTool.connection.serverURL,
            checkerURL: CheckerTool.connection.checkerURL,
            computeURL: CheckerTool.connection.computeURL
        };
        this.addStyles();
        this.initHandlers();
    };

    /**
     * Init event handlers
     */
    CheckerTool.prototype.initHandlers = function() {
        let tool = this;
    };

    /**
     * Get the configs
     */
    CheckerTool.prototype.getConfigs = function() {
        $('.virumcourse-wrapper').trigger('getconfigs', {appname: 'checker'});
    };

    /**
     * Get connection urls
     */
    CheckerTool.prototype.getConnections = function() {
        $('.virumcourse-wrapper').trigger('getconnections', {appname: 'checker'});
    };

    /**
     * Get user info
     */
    CheckerTool.prototype.getUserinfo = function() {
        // First try to get separate checkeruser (Works on 4f Note)
        let userinfo = JSON.parse(JSON.stringify(CheckerTool.user));
        $('.virumcourse-wrapper').trigger('getcheckeruser', [userinfo]);
        // Put the user object back, updated or not.
        if (userinfo.appkey) {
            userinfo.applicationkey = userinfo.appkey;
            delete userinfo.appkey;
        } else {
            userinfo.applicationkey = '';
            userinfo.username = '';
        };
        CheckerTool.user = userinfo;
        // Try to get the userinfo from the context.
        $('.virumcourse-wrapper').trigger('getuserinfo', {appname: 'checker'});
    };

    /**
     * Call checker endpoint
     */
    CheckerTool.prototype.callChecker = function(endpoint, targetLoc, signatureData) {
        const tool = this;
        const socket = new WebSocket(endpoint);

        var cleanupCompleted = false;
        var wrongSteps = [];
        var derivationError = false;
        var validData = {
            checkdata: this.checkData
        };
        if (signatureData != null) {
            validData['signature'] = signatureData.signature;
        }
        if (targetLoc != null) {
            validData['targetLoc'] = targetLoc.join('_');
        }

        this.result = {
            general: 'General feedback for whole solution',
            messages: []
        };

        let checkingCompleted = function() {
            if (!cleanupCompleted) {
                // Removes the progress objects from the message array.
                tool.removeProgress();
                tool.clearMarkings();
                tool.curtainRemove();
                if (!derivationError && endpoint !== tool.settings.computeURL) {
                    validData.checkdata.steps.forEach(function(value, index) {
                        if (value.steptype !== 'assumption' && value.type !== 'declaration') {
                            // Checks if the related derivation has been marked to unproved list or not.
                            if (wrongSteps.indexOf(value.loc) === -1) {
                                tool.setMessage('correct', 'correct', value.loc);
                            } else {
                                tool.setMessage('wrong', 'wrong', value.loc);
                            };
                        };
                    });
                };
                // Render the checker box with updated results, if it exists.
                $('#checkerbox').html(JSON.stringify(tool.result, null, 4));
                tool.addMarkings();

                console.log('Checking completed');
                cleanupCompleted = true;
            }
         };

        //// On error
        socket.onerror = function(e) {
            derivationError = true;
            console.log("Unable to open websocket: ", e);
            tool.setMessage('wrong', 'serverfailure');
            socket.close();
        };

        //// On Open
        socket.onopen = function(e) {
            // If there is "#checkersendbox", then copy the checkdata into that.
            $('#checkersendbox').val(JSON.stringify(validData, null, 4));
            try {
                // Put progress message to the derivation feedback box for each task and observation.
                validData.checkdata.steps.forEach(function(value, index) {
                    if (value.steptype !== 'assumption' && value.type !== 'declaration') {
                        tool.setMessage('wrong', 'progress', value.loc);
                    };
                });
                tool.clearMarkings();
                tool.addMarkings();
                socket.send(JSON.stringify(validData));
            } catch (err) {
                // If derivation list (steps = []) is empty. Meaning nothing to prove.
                if (validData.checkdata.steps.length === 0) {
                    tool.setMessage('correct', 'empty');
                };
                socket.close();
            };
        };

        //// On message
        socket.onmessage = function(e) {
            const receivedData = JSON.parse(e.data);
            /*
             * Receive 'checker' -> all communication over -> explicitly close the socket. This is to work around an
             * issue in Chromium where it takes around 2 seconds before onclose is fired (wss only).
             */
            if (receivedData.mark === 'checker') {
                checkingCompleted();
            } else {
                /*
                 * Checks if the location is empty which occurs when derivation format is wrong or java error on backend
                 * and also checks if the error is of missing value type. Such as missing relation.
                 */
                if (receivedData.loc === '' || receivedData.loc === null) {
                    derivationError = true;
                    tool.setMessage(receivedData.mark, receivedData.comment);
                    return; // Should not append the result messages with null location message.
                } else if (receivedData.mark === 'missingValue') {
                    derivationError = true;
                    receivedData.mark = 'wrong';
                    tool.setMessage('wrong', 'derivationError');
                } else if (receivedData.mark === 'wrong' || receivedData.mark === 'note') {
                    wrongSteps.push(receivedData.loc.split('_').slice(0, 3).join('_'));
                } else if (receivedData.mark === 'computation') {
                    let relsym = receivedData.relation;
                    let result = receivedData.comment;
                    let locType = targetLoc[targetLoc.length - 2];
                    if (locType === 'term') {
                        let newTerm = CheckerTool.normalizeMQLatex(result);
                        console.log(targetLoc);
                        let termIndex = targetLoc[targetLoc.length - 1];
                        let nextIndex = (termIndex | 0) + 1;
                        let tloc = targetLoc.slice(0,targetLoc.length - 1).concat(nextIndex.toString());
                        let motivation = tool.getMessage('computed');
                        if (tool.editor.locationExists(tloc.slice(0))) {
                            let rloc = targetLoc.slice(0,targetLoc.length - 2).concat(['relation',termIndex.toString()]);
                            tool.editor.doEvent(tool.editor.createEvent('edit', rloc.slice(0), [{text: relsym},{}]));
                            let mloc = targetLoc.slice(0,targetLoc.length - 2).concat(['motivation',termIndex.toString()]);
                            tool.editor.doEvent(tool.editor.createEvent('edit', mloc.slice(0), [{text: motivation},{}]));
                            tool.editor.doEvent(tool.editor.createEvent('edit', tloc.slice(0), [{text: newTerm},{}]));
                        } else {
                            tool.editor.doEvent(tool.editor.createEvent('addstep', targetLoc.slice(0), [
                                {
                                    relation: { text: relsym, readonly: false, attrs: {} },
                                    motivation: { text: motivation, readonly: false, attrs: {} },
                                    term: { text: newTerm, readonly: false, attrs: {} }
                                },
                                {}
                            ]));
                        }
                         tool.editor.show(tloc);
                     }
                     return; // Should not show marker
                }
                tool.setMessage(receivedData);
                tool.addMarkings();
            }
        };

        //// On close
        socket.onclose = function(e) {
            if (!cleanupCompleted) {
                // we received onclose before the hangup message -> server probably crashed, so show error status
                derivationError = true;
                console.log("Websocket closed before hangup message received");
                tool.setMessage('wrong', 'serverfailure');
                checkingCompleted();
            }
        }
    };

    /**
     * Sign data and pass to checker (for checking or computing)
     */
    CheckerTool.prototype.signRequestAndCallChecker = function(endpoint, targetLoc) {
        // Sign the data and then send it to the checker.
        let tool = this;

        this.curtainAdd();
        try {
            if (tool.settings.username != null && tool.settings.appkey != null) {
                // retrieve signature for the request from the backend server
                $.ajax({
                    type: 'POST',
                    url: tool.settings.serverURL,
                    dataType: 'json',
                    timeout: 5000,
                    data: {
                        type: 13,
                        username: tool.settings.username,
                        appkey: tool.settings.appkey,
                        mode: endpoint == this.settings.computeURL ? "compute" : "check",
                        json: JSON.stringify(tool.checkData)
                    }
                }).done(function(respondData) {
                    if (respondData.success) {
                        tool.callChecker(endpoint,targetLoc,respondData);
                    } else {
                        tool.curtainRemove();
                        tool.failNotice('unauthorized');
                    };
                }).fail(function(failData) {
                    tool.curtainRemove();
                    tool.failNotice('authservererror', failData);
                });
            } else {
                // else call without signed request, for local testing
                tool.callChecker(endpoint,targetLoc,null);
            }
        } catch (err) {
            console.log('ERROR', err);
            tool.curtainRemove();
        };
    };

    CheckerTool.prototype.check = function() {
        this.signRequestAndCallChecker(this.settings.checkerURL,null);
    }

    CheckerTool.prototype.compute = function(targetLoc) {
        this.signRequestAndCallChecker(this.settings.computeURL,targetLoc);
    }

    /**
     * Inform user about failure
     * @param {String} failtype     Type of failure ('unauthorized'|'authservererror')
     * @param {Option} faildata     Optional faildata
     */
    CheckerTool.prototype.failNotice = function(failtype, faildata) {
        this.clearMarkings();
        this.addMarkings({
            general: 'General feedback for whole solution',
            messages: [{
                loc: this.checkname,
                mark: 'wrong',
                comment: this.getMessage(failtype)
            }]
        });
        if (faildata) {
            console.log('error', faildata);
        };
    };

    /**
     * Set message to the result
     * @param {String|Object} type    Type of feedback or object of format: {loc: '', mark: '', comment: ''}
     * @param {String} msgid          Message id
     * @param {String} loc            Optional location. If not given, loc is this.checkname (general feedback)
     */
    CheckerTool.prototype.setMessage = function(type, msgid, loc) {
        if (typeof(type) === 'object') {
            loc = type.loc;
            msgid = type.comment;
            type = type.mark;
        };
        this.result.messages.push({
            loc: loc || this.checkname,
            mark: type,
            comment: this.getMessage(msgid)
        });
    };

    /**
     * Remove progress messages
     */
    CheckerTool.prototype.removeProgress = function() {
        let progressmsg = this.getMessage('progress');
        this.result.messages = this.result.messages.filter(function(value) {
            return value.comment !== progressmsg;
        });
    };

    /**
     * Clear all markings
     */
    CheckerTool.prototype.clearMarkings = function() {
        this.editor.place.trigger('clearreview');
    };

    /**
     * Set editor markings
     * @param {Object} markings    Markings for the editor
     */
    CheckerTool.prototype.addMarkings = function(markings) {
        markings = markings || this.result || [];
        this.editor.place.trigger('addmarkings', markings);
    };

    /**
     * Get the html for the curtain
     * @returns {String}   html for curtain
     */
    CheckerTool.prototype.getCurtain = function() {
        return `<div class="sdCheckCurtain">${this.waitanimation}</div>`;
    };

    /**
     * Add curtain
     */
    CheckerTool.prototype.curtainAdd = function() {
        this.curtain = $(this.getCurtain());
        this.editor.place.append(this.curtain);
    };

    /**
     * Remove curtain
     */
    CheckerTool.prototype.curtainRemove = function() {
        this.curtain.remove();
    };

    /**
     * Get localized message
     * @param {String} msgid    The message id
     * @returns {String}        The localized message
     */
    CheckerTool.prototype.getMessage = function(msgid) {
        let uilang = this.uilang;
        let langdata = this.messages[uilang] && this.messages['en'];
        let message = langdata[msgid] || this.messages['en'][msgid] || msgid;
        return message;
    };

    /**
     * Set style tag to the head of the document, if it's not there already.
     */
    CheckerTool.prototype.addStyles = function() {
        if ($('#sdeditor-checker-css').length === 0) {
            $('head').append(`
                <style id="sdeditor-checker-css" type="text/css">
                ${this.styles}
                </style>
            `);
        };
    };

    /**
     * Default values
     */
    CheckerTool.prototype.defaults = {
        username: 'Anonymous',
        lang: 'en',
        decimalSeparator: '.',
        declarations: [
              {
                "declaration": {
                  "expr": "$P,Q,R: \\BB$",
                  "loc": ""
                },
                "definition": {
                  "expr": "",
                  "loc": ""
                }
              },              {
                "declaration": {
                  "expr": "$a,b,c,d,p,t,x,y,z: \\RR$",
                  "loc": ""
                },
                "definition": {
                  "expr": "",
                  "loc": ""
                }
              },
              {
                "declaration": {
                  "expr": "$f,g: \\RR \\to \\RR$",
                  "loc": ""
                },
                "definition": {
                  "expr": "",
                  "loc": ""
                }
              },
              {
                "declaration": {
                  "expr": "$i,j,k,m,n: \\ZZ$",
                  "loc": ""
              },
                "definition": {
                  "expr": "",
                  "loc": ""
              }
            }
         ]
    };

    /**
     * Localized messages.
     */
    CheckerTool.prototype.messages = {
        en: {
            'unauthorized': 'The user has depleted the checking quota.',
            'authservererror': 'The authentication server is not responding.',
            'empty': 'Nothing to check.',
            'correct': 'The derivation is correct.',
            'wrong': 'Some steps could not be automatically verified.',
            'progress': 'Checking is in progress...',
            'serverfailure': 'Server error or too many requests. Please try again later.',
            'derivationError': 'The derivation has errors and hence could not be checked.',
            'computed': 'computed'
        },
        fi: {
            'unauthorized': 'Käyttäjä on kuluttanut tarkistuskiintiön loppuun.',
            'authservererror': 'Todennuspalvelin ei vastaa.',
            'empty': 'Ei mitään tarkistettavaa.',
            'correct': 'Päättelyketju on oikein.',
            'wrong': 'Joitakin askeleita ei voitu automaattisesti tarkistaa.',
            'progress': 'Tarkistaminen on käynnissä...',
            'serverfailure': 'Palvelimessa virhe tai liian monta pyyntöä. Kokeile uudestaan myöhemmin.',
            'derivationError': 'Päättelyketjussa on virheitä, joten sitä ei voi tarkistaa.',
            'computed': 'laskettu'
        },
        sv: {
            'unauthorized': 'Användaren har använt upp sin checkningskvot.',
            'authservererror': 'Autenticeringsservern svarar inte.',
            'empty': 'Inget att granska.',
            'correct': 'Härledningen är korrekt.',
            'wrong': 'Vissa steg kan inte verifieras automatiskt.',
            'progress': 'Granskning pågår...',
            'serverfailure': 'Serverfel eller för många förfrågningar. Var god försök igen senare.',
            'derivationError': 'Härledningen innehåller fel som gör att den inte kunde granskas.',
            'computed': 'uträknat'
        }
    }

    CheckerTool.prototype.icons = {
        check: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="20" height="20" viewBox="0 0 20 20" class="mini-icon mini-icon-check"><path stroke="none" fill="#666" d="M3 10 l3 0 l3 5 l7 -12 l2 2 l-9 14z" /><path stroke="none" fill="#070" d="M2 9 l3 0 l3 5 l7 -12 l2 2 l-9 14z" /></svg>'
    };

    CheckerTool.prototype.waitanimation = '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="100" height="100" viewBox="0 0 100 100" class="sdeditor-icon sdeditor-icon-wait"><circle stroke="none" fill="#666" cx="50" cy="20" r="0"><animate attributeType="XML" attributeName="r" begin="0.00s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="0.00s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="70" cy="30" r="0"><animate attributeType="XML" attributeName="r" begin="0.375s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="0.375s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="80" cy="50" r="0"><animate attributeType="XML" attributeName="r" begin="0.75s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="0.75s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="70" cy="70" r="0"><animate attributeType="XML" attributeName="r" begin="1.125s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="1.125s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="50" cy="80" r="0"><animate attributeType="XML" attributeName="r" begin="1.50s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="1.50s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="30" cy="70" r="0"><animate attributeType="XML" attributeName="r" begin="1.875s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="1.875s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="20" cy="50" r="0"><animate attributeType="XML" attributeName="r" begin="2.25s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="2.25s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle><circle stroke="none" fill="#666" cx="30" cy="30" r="0"><animate attributeType="XML" attributeName="r" begin="2.625s" dur="3s" values="0; 8; 0; 0;" repeatCount="indefinite" /><animate attributeType="XML" attributeName="opacity" begin="2.625s" dur="3s" values="0; 1; 0; 0;" repeatCount="indefinite" /></circle></svg>';

    CheckerTool.prototype.styles = `
        .sdCheckCurtain {
            position: absolute;
            top: 0; bottom: 0; left: 0; right: 0;
            background-color: rgba(255,255,255,0.4);
        }
        .sdCheckCurtain svg {
            position: absolute;
            top: 50%; left: 50%;
            margin-top: -50px;
            margin-left: -50px;
        }
    `;

    const TutorialFileName = "checker-tutorial";
    const TutorialContentId = 'notebook-4fnotes-0';

    // A plugins-object can include several plugins with different keys.
    //---------------------------------------------------------------------
    let plugins = {
        /*********************************
         * Checker plugin
         *********************************/
        Checker: {
            // Label of the menu
            label: {
                en: 'Check',
                fi: 'Tarkista',
                sv: 'Granska'
            },
            // Plugins are ordered by this "weight" in the menu.
            order: 1,

            // This is executed, when the plugin is drawn in the editor
            init: function(editor) {
                let uilang = editor.settings.uilang;
                CheckerTool.registerConfigs(uilang);
                CheckerTool.registerKeybindings(editor);
            },

            eventHandler: function(editor, eventName, mouseEvent) {
                if (eventName==='compute' && mouseEvent.type==='mousedown' && mouseEvent.button==0) {
                    CheckerTool.computeHandler(editor);
                }
            },

            // Menu items that are selectable from this plugin.
            items: [{
                // Name of the menu item.
                name: 'Checker',

                // Label of the menu item in different languages.
                label: {
                    en: 'Check',
                    fi: 'Tarkista',
                    sv: 'Granska'
                },

                // Icon shown in the menu (html5-string, can be empty string).
                icon: '<i class="fa fa-check-square" style="width: 1em; color:green"></i>',

                // Description shown as a tooltip for the menu item.
                description: {
                    en: 'Check the derivation (Ctrl+!)',
                    fi: 'Tarkista päättelyketju (Ctrl+!)',
                    sv: 'Granska härledningen (Ctrl+!)'
                },

                // Function that will be executed, when this menu item is selected.
                // Gets the editor-object as parameter.
                action: function(editor) {
                    let checkertool = new CheckerTool(editor);
                    checkertool.init();
                    checkertool.check();
                }
            },
            {
                // This menu item clears the review markings from the derivation.
                name: 'Clear markings',
                label: {
                    en: 'Clear markings',
                    fi: 'Poista merkinnät',
                    sv: 'Avlägsna markeringarna'
                },
                icon: '<i class="fa fa-eraser" style="width: 1em; color: gray"></i>',

                description: {
                    en: 'Clear the correction markings',
                    fi: 'Poista korjausmerkinnät',
                    sv: 'Avlägsna korrigeringsmarkeringarna'
                },
                action: function(editor) {
                    // Just set empty review.
                    editor.place.trigger('clearreview');
                    //curtain.remove();
                }
            },
            {
                // This menu opens the settings dialog
                name: 'Settings',
                label: {
                    en: 'Settings',
                    fi: 'Asetukset',
                    sv: 'Inställningar'
                },
                icon: '<i class="fa fa-cog" style="width: 1em; color: gray"></i>',
                description: {
                    en: 'Settings',
                    fi: 'Asetukset',
                    sv: 'Inställningar'
                },
                action: function(editor) {
                    editor.place.trigger('openconfig', 'checker');
                }
            }],
            buttons: [
                {
                    // Undo
                    name: 'compute',
                    title: {
                        en: 'Compute the value of the expression (Ctrl+↵)',
                        fi: 'Laske lausekkeen arvo (Ctrl+↵)',
                        sv: 'Beräkna uttryckets värde (Ctrl+↵)'
                    },
                    event: 'compute',
                    text: '=',
                    icon: '<i class="fa fa-calculator" style="font-size: 10px; color: blue"></i>',
                }
            ]
        },

        /*********************************
         * Help menu
         *********************************/
        Help: {
            // Label of the menu
            label: {
                en: 'Help',
                fi: 'Ohje',
                sv: 'Hjälp'
            },
            // Plugins are ordered by this "weight" in the menu.
            order: 3,

            // This is executed, when the plugin is drawn in the editor
            init: function(editor) {},

            // Menu items that are selectable from this plugin.
            items: [
                {
                    name: 'Using4fChecker',
                    label: {
                        en: 'Using 4f Checker',
                        fi: '4f Checkerin käyttö',
                        sv: 'Att använda 4f Checker'
                    },
                    icon: '<i class="fa fa-bookmark" style="width: 1em; color: orange"></i>',
                    description: {
                        en: 'Open chapter "Using 4f Checker"',
                        fi: 'Avaa luku "Using 4f Checker"',
                        sv: 'Öppna kapitel "Using 4f Checker"'
                    },
                    action: function(editor) {
                        editor.place.trigger('startTutorial.system',
                            {
                                filename: TutorialFileName,
                                contentId: TutorialContentId,
                                plugins: [
                                    {
                                        pluginName: 'notebook',
                                        params: {
                                            systemRequests: [{type: 'getContentAsObject'}]
                                        }
                                    }
                                ],
                                link: {
                                    chapid: '4fnotesuserfi20180509T074252713Z',
                                    elementsetpath: []
                                },
                                menu: []
                            });
                    }
                },
                                {
                    name: 'Terms',
                    label: {
                        en: 'Terms',
                        fi: 'Lausekkeet',
                        sv: 'Uttryck'
                    },
                    icon: '<i class="fa fa-bookmark" style="width: 1em; color: orange"></i>',
                    description: {
                        en: 'Open chapter "Terms"',
                        fi: 'Avaa luku "Terms"',
                        sv: 'Öppna kapitel "Terms"'
                    },
                    action: function(editor) {
                        editor.place.trigger('startTutorial.system',
                            {
                                filename: TutorialFileName,
                                contentId: TutorialContentId,
                                plugins: [
                                    {
                                        pluginName: 'notebook',
                                        params: {
                                            systemRequests: [{type: 'getContentAsObject'}]
                                        }
                                    }
                                ],
                                link: {
                                    chapid: '4fnotesuserfi20180329T082614028Z',
                                    elementsetpath: []
                                },
                                menu: []
                            });
                    }
                },
                {
                    name: 'DeclarationsAndDefinitions',
                    label: {
                        en: 'Declarations and definitions',
                        fi: 'Deklaraatiot ja määritelmät',
                        sv: 'Deklarationer och definitioner'
                    },
                    icon: '<i class="fa fa-bookmark" style="width: 1em; color: orange"></i>',
                    description: {
                        en: 'Open chapter "Declarations and definitions"',
                        fi: 'Avaa luku "Declarations and definitions"',
                        sv: 'Öppna kapitel "Declarations and definitions"'
                    },
                    action: function(editor) {
                        editor.place.trigger('startTutorial.system',
                            {
                                filename: TutorialFileName,
                                contentId: TutorialContentId,
                                plugins: [
                                    {
                                        pluginName: 'notebook',
                                        params: {
                                            systemRequests: [{type: 'getContentAsObject'}]
                                        }
                                    }
                                ],
                                link: {
                                    chapid: '4fnotesuserfi20180406T081024810Z',
                                    elementsetpath: []
                                },
                                menu: []
                            });
                    }
                },
                {
                    name: 'JustificationDirectives',
                    label: {
                        en: 'Justification directives',
                        fi: 'Perustelu-direktiivit',
                        sv: 'Motiverings-direktiv'
                    },
                    icon: '<i class="fa fa-bookmark" style="width: 1em; color: orange"></i>',
                    description: {
                        en: 'Open chapter "Justification directives"',
                        fi: 'Avaa luku "Justification directives"',
                        sv: 'Öppna kapitel "Justification directives"'
                    },
                    action: function(editor) {
                        editor.place.trigger('startTutorial.system',
                            {
                                filename: TutorialFileName,
                                contentId: TutorialContentId,
                                plugins: [
                                    {
                                        pluginName: 'notebook',
                                        params: {
                                            systemRequests: [{type: 'getContentAsObject'}]
                                        }
                                    }
                                ],
                                link: {
                                    chapid: '4fnotesuserfi20180406T082803997Z',
                                    elementsetpath: []
                                },
                                menu: []
                            });
                    }
                },
                {
                    name: 'CurrentLimitations',
                    label: {
                        en: 'Current limitations',
                        fi: 'Rajoitukset',
                        sv: 'Begräsningar'
                    },
                    icon: '<i class="fa fa-bookmark" style="width: 1em; color: orange"></i>',
                    description: {
                        en: 'Open chapter "Current limitations"',
                        fi: 'Avaa luku "Current limitations"',
                        sv: 'Öppna kapitel "Current limitations"'
                    },
                    action: function(editor) {
                        editor.place.trigger('startTutorial.system',
                            {
                                filename: TutorialFileName,
                                contentId: TutorialContentId,
                                plugins: [
                                    {
                                        pluginName: 'notebook',
                                        params: {
                                            systemRequests: [{type: 'getContentAsObject'}]
                                        }
                                    }
                                ],
                                link: {
                                    chapid: '4fnotesuserfi20180514T135300642Z',
                                    elementsetpath: []
                                },
                                menu: []
                            });
                    }
                }
            ]
         }
    };

    // Load this plugin to the sdeditor.
    $.fn.sdelement('addplugin', plugins);

    // Register checker tool for config dialog.
    CheckerTool.registerConfigs();

    CheckerTool.addStyles();

})(jQuery)
