/******
@name Contextboard
@version 0.1
@author Petri Salmela <petri.salmela@fourferries.fi>
@type plugin
@requires jQuery x.x.x or newer
@class Contextboard
@description A class and jQuery-plugin for a user interface for modifying context details such as name, description, members, etc.

TODO:
*******/

/**
 * Requirements:
 * - jQuery
 */

try {
    typeof(jQuery) !== 'undefined';
} catch (err) {
    throw new Error('Missing dependency in ' + err.fileName + '\n' + err);
}

/**
 * Optional requirements
 * - EbookLocalizer
 * - Elpp-libs
 */

if (typeof(checkOptionalRequirements) !== 'undefined' && checkOptionalRequirements) {
    try {
        typeof(EbookLocalizer) !== 'undefined';
        typeof(ebooklocalizer) !== 'undefined';
        typeof(Elpp) === 'undefined' && Elpp.apply;
    } catch (err) {
        throw new Error('Missing optional dependency in ' + err.fileName + '\n' + err);
    }
}

;(function(window, $, ebooklocalizer){

    /**
     * Helper functions
     */

    /**
     * Sanitize text
     * @param {String} text    Text to sanitize
     * @param {Object} options Options for sanitizer
     */
    var sanitize = function(text, options) {
        options = $.extend({
            SAFE_FOR_JQUERY: true
        }, options);
        return DOMPurify.sanitize(text, options);
    };

    /**
     * Escape html for security
     */
    var escapeHTML = function(html) {
        return document.createElement('div')
            .appendChild(document.createTextNode(html))
            .parentNode
            .innerHTML
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
    };

    /**
     * Context board
     * @constructor
     * @param {jQuery} place    DOM-place for course dashboard
     * @param {Object} options  data for course dashboard
     */
    var Contextboard = function(place, options) {
        options = $.extend(true, {}, this.defaults, options);
        this.place = $(place);
        this.init(options);
        this.initHandlers();
        this.initData();
        this.setAttrs();
        this.setStyles();
        this.show();
        this.place.trigger('addguidedtour', Contextboard.tourguide);
    }

    /**
     * Init the tool
     * @param {Object} options  Data to init this tool
     */
    Contextboard.prototype.init = function(options) {
        this.settings = options.settings;
        this.users = this.settings.users;
        this.appid = this.settings.appid;
        this.apptype = this.settings.apptype;
        this.contextid = this.settings.context.id;
        this.contexttype = this.settings.context.type;
    };

    /**
     * Init the data. (Schools, subjects etc.)
     */
    Contextboard.prototype.initData = function() {
        this.initSchools();
        var subjects = this.settings.libraryinfo && this.settings.libraryinfo.subjects || [];
        this.subjects = new Elpp.SubjectSet(subjects);
        //this.place.trigger('getcontent', {anchors: [this.settings.contextid], contentType: ['hamessage', 'solution', 'review']});
    };

    /**
     * Init the map of schools by id.
     */
    Contextboard.prototype.initSchools = function() {
        this.schoolsById = {};
        var schools = this.settings.libraryinfo && this.settings.libraryinfo.schools || [];
        var sid;
        for (let i = 0, len = schools.length; i < len; i++) {
            sid = schools[i].id;
            this.schoolsById[sid] = schools[i];
        };
    };

    /**
     * Init the event handlers.
     */
    Contextboard.prototype.initHandlers = function() {
        var board = this;
        //this.place.on('reply_getcontent', function(event, data) {
        //    event.stopPropagation();
        //    board.setContentdata(data);
        //});
        this.place.off('click', '.contextboard-messagelink').on('click', '.contextboard-messagelink', function(event) {
            event.stopPropagation();
            var toid = $(this).attr('data-toid');
            board.composeMessage(toid);
        });
        this.place.on('refreshapp', function(event, data) {
            event.stopPropagation();
            board.initData();
            board.show();
        });
        this.place.on('closechildrenapp', function(event, data){
            event.stopPropagation();
            board.close();
        });
        this.place.on('change', 'input.contextboard-users-selectall', function(event, data) {
            event.stopPropagation();
            let input = $(this);
            let selected = input.is(':checked');
            let gid = input.attr('data-gid');
            board.toggleAllUsers(gid, selected);
        }).on('change', 'input.contextboard-users-selectuser', function(event, data) {
            event.stopPropagation();
            let input = $(this);
            let selected = input.is(':checked');
            let gid = input.closest('.contextboard-usertable').attr('data-gid');
            board.updateSelectAll(gid, selected);
        }).on('click', '.contextboard-actionbutton', function(event, data) {
            event.stopPropagation();
            let button = $(this);
            let action = button.attr('data-action');
            let groupid = button.closest('.contextboard-usergroup').attr('data-gid');
            switch (action) {
                case 'removeuser':
                    board.userManagementRemove(groupid);
                    break;
                case 'adduser':
                    let username = button.closest('tr').find('input.contextboard-addusername').val();
                    board.userManagementAdd(username, groupid);
                    break;
                case 'interrupt_send':
                    board.stopServerWait({});
                    break;
                default:
                    break;
            };
        }).on('keypress', 'input.contextboard-addusername', function(event, data) {
            event.stopPropagation();
            if (event.which === 13) {
                $(this).closest('tr').find('.contextboard-actionbutton').click();
            };
        }).on('relay_reply_usermanagement', function(event, data) {
            event.stopPropagation();
            let actions = data.data.actions;
            if (data.success) {
                board.modifyUsers(actions);
            };
            board.stopServerWait(data);
            board.requestContextinfoUpdate();
        }).on('updatedusers', function(event, data) {
            event.stopPropagation();
            board.users = data;
            board.view();
        });
    };

    /**
     * Set html attributes, classes etc.
     */
    Contextboard.prototype.setAttrs = function() {
        this.place.addClass('contextboard');
    };

    /**
     * Show the tool
     */
    Contextboard.prototype.show = function() {
        this.view();
    };

    /**
     * Show the tool in view mode.
     */
    Contextboard.prototype.view = function() {
        this.place.html(this.getTemplate('html'));
        this.viewInfo();
        if (this.users.canEdit({ctype: 'notebookcontent', contextid: this.settings.contextid})) {
            this.viewUserList();
        };
        this.setFocus();
    };

    /**
     * Show the information about the context. (Subject, teacher, description, etc.)
     */
    Contextboard.prototype.viewInfo = function() {
        var uilang = this.settings.uilang;
        var context = this.settings.context;
        var place = this.place.children('.contextboard-contextinfo');
        place.empty();
        var html = ['<table class="contextboard-infotable">', '<tbody>'];
        html.push(`<tr><td>${this.getSubjectIcons()}</td><th>${sanitize(context.titles[uilang] || context.titles.common || '')}</th></tr>`);
        var provider = '';
        if (context.providing_school) {
            provider = (this.schoolsById[context.providing_school] || {name: ''}).name;
            provider = provider ? '(' + provider + ')' : '';
        }
        if (this.settings.context.teacher) {
            html.push(`<tr><td class="contextboard-infolabel"><span class="contextboard-info-teacher">${ebooklocalizer.localize('contextboard:teacher', uilang)}:</td><td>${sanitize(this.users.getRealname(this.settings.context.teacher.username || ''))}</span> <span class="contextboard-info-school">${provider}</span></td></tr>`);
        };
        if (context.start_time ) {
            let time = (new Date(context.start_time)).toLocaleDateString(uilang);
            html.push(`<tr><td class="contextboard-infolabel">${ebooklocalizer.localize('contextboard:starttime', uilang)}:</td><td>${sanitize(time)}</td></tr>`)
        };
        if (context.end_time ) {
            let time = (new Date(context.end_time)).toLocaleDateString(uilang);
            html.push(`<tr><td class="contextboard-infolabel">${ebooklocalizer.localize('contextboard:endtime', uilang)}:</td><td>${sanitize(time)}</td></tr>`)
        };
        html.push(`<tr><td class="contextboard-infolabel">${ebooklocalizer.localize('contextboard:shortdescription', uilang)}:</td><td>${sanitize(context.short_desc)}</td></tr>`);
        html.push(`<tr><td class="contextboard-infolabel">${ebooklocalizer.localize('contextboard:description', uilang)}:</td><td>${sanitize(context.description)}</td></tr>`);
        html.push('</tbody>', '</table>');
        place.html(html.join('\n'));
    };

    /**
     * View the lists of users.
     */
    Contextboard.prototype.viewUserList = function() {
        let cboard = this;
        let uilang = this.settings.uilang;
        let me = this.users.username;
        let place = this.place.children('.contextboard-userlist');
        place.empty();
        let html = {};
        let groups = this.users.getGroups().sort();
        for (let i = 0, len = groups.length; i < len; i++) {
            let gid = groups[i];
            html[gid] =  [
                `<div class="contextboard-usergroup" data-gid="${sanitize(gid)}">`,
                `<table class="contextboard-usertable" data-gid="${sanitize(gid)}">`,
                `<caption>${this.users.groups[gid].getDescription()}</caption>`,
                '<thead class="ffwidget-background-colored">',
                `<tr>
                <th class="contextboard-selectcell"><input type="checkbox" class="contextboard-users-selectall" data-gid="${sanitize(gid)}"></th>
                <th>${ebooklocalizer.localize('contextboard:name', uilang)}</th>
                <th>${ebooklocalizer.localize('contextboard:username', uilang)}</th>
                <th>${ebooklocalizer.localize('contextboard:email', uilang)}</th>
                <th>${ebooklocalizer.localize('contextboard:school', uilang)}</th>
                </tr>`,
                '</thead>',
                '<tbody>'
            ];
        };
        let userlist = this.users.getUserList(groups);
        for (let i = 0, len = userlist.length; i < len; i++) {
            let user = userlist[i];
            let isme = me === user.getUsername();
            let schools = user.getSchools();
            let schoolnames = schools.map(function(id) {return cboard.schoolsById[id].name});
            let row = `
                <tr data-username="${escapeHTML(user.getUsername())}">
                    <td class="contextboard-selectcell"><input type="checkbox" class="contextboard-users-selectuser" data-user="${sanitize(user.getUsername())}" ${isme ? 'disabled' : ''}></td>
                    <td data-userdata="realname">${sanitize(user.getRealnameLnameFirst())}</td>
                    <td data-userdata="username">${sanitize(user.getUsername())} ${this.getMessageLink(user.getUsername())}</td>
                    <td data-userdata="email"><a href="mailto:${encodeURIComponent(user.data.email)}">${sanitize(user.getEmail())}</a></td>
                    <td data-userdata="schools">${sanitize(schoolnames.join(',<br>'))}</td>
                </tr>
            `;
            let ugroups = user.getGroups();
            for (let j = 0, glen = ugroups.length; j < glen; j++) {
                html[ugroups[j]].push(row);
            };
        };
        for (let i = 0, len = groups.length; i < len; i++) {
            let gid = groups[i];
            html[gid].push(`
                </tbody>
                <tfoot>
                    <tr>
                        <td colspan="2"><button class="ffwidget-button contextboard-actionbutton" data-action="adduser">${ebooklocalizer.localize('contextboard:add', uilang)}</button></td>
                        <td><input type="text" class="contextboard-addusername" placeholder="${ebooklocalizer.localize('contextboard:username', uilang)}"></td>
                        <td></td>
                        <td></td>
                    </tr>
                    <tr>
                        <td colspan="2"><button class="ffwidget-button contextboard-actionbutton" data-action="removeuser">${ebooklocalizer.localize('contextboard:remove', uilang)}</button></td>
                        <td></td>
                        <td></td>
                        <td></td>
                    </tr>
                </tfoot>
                </table>
                </div>
            `)
            place.append(html[gid].join('\n'));
        };
    };

    /**
     * Get the icons for the subjects.
     * @returns {String}   Html for the list of subject icons.
     */
    Contextboard.prototype.getSubjectIcons = function() {
        var lang = this.settings.uilang;
        var result = ['<ul class="contextboard-subjectlist">'];
        var subjects = this.settings.context.subjects || [];
        var subj;
        for (let i = 0, len = subjects.length; i < len; i++) {
            subj = subjects[i];
            result.push(`
                <li>
                    <div class="contextboard-subjecticon">${this.subjects.getIcon(subj)}</div>
                    <div class="contextboard-subjecttext">${this.subjects.getTitle(subj, lang)}</div>
                </li>
            `);
        };
        result.push('</ul>');
        return result.join('\n');
    };

    /**
     * Get the message link for the user.
     * @param {String} username   The username of the user.
     * @returns {String}   Html for the link.
     */
    Contextboard.prototype.getMessageLink = function(username) {
        var html = `<span class="contextboard-messagelink" data-toid="${sanitize(username)}">${this.icons.message}</span>`;
        return html;
    };
    
    /**
     * Toggle all users (selected/unselected)
     * @param {String} gid   Group id
     * @param {Boolean} selected   Selected or not.
     */
    Contextboard.prototype.toggleAllUsers = function(gid, selected) {
        selected = !!selected;
        let inputs = this.place.find(`.contextboard-usertable[data-gid="${sanitize(gid)}"] input.contextboard-users-selectuser:not(:disabled)`);
        if (selected) {
            inputs.prop('checked', true);
        } else {
            inputs.prop('checked', false);
        };
    };
    
    /**
     * Update "select all" checkbox, if needed
     * @param {String} gid   Group id
     * @param {Boolean} selected    Is the clicked checkbox on or off
     */
    Contextboard.prototype.updateSelectAll = function(gid, selected) {
        selected = !!selected;
        let selectall = this.place.find(`.contextboard-usertable[data-gid="${sanitize(gid)}"] input.contextboard-users-selectall`);
        if (!selected) {
            selectall.prop('checked', false);
        } else {
            let inputs = this.place.find(`.contextboard-usertable[data-gid="${sanitize(gid)}"] input.contextboard-users-selectuser:not(:disabled)`);
            let allins = this.place.find(`.contextboard-usertable[data-gid="${sanitize(gid)}"] input.contextboard-users-selectuser:not(:disabled):checked`);
            if (inputs.length === allins.length) {
                selectall.prop('checked', true);
            };
        };
    };

    /**
     * Start message composing to given user.
     * @param {String} toid    The id of the user.
     */
    Contextboard.prototype.composeMessage = function(toid) {
        var data = {
            toid: toid,
            totype: 'user',
            contentdata: {
                type: 'markdownelement',
                metadata: {
                    creator: this.settings.username,
                    modifier: this.settings.username,
                    created: (new Date()).getTime(),
                    modified: (new Date()).getTime(),
                    tags: []
                },
                data: {
                    text: ''
                }
            }
        };
        this.place.trigger('composemessagedata', [data]);
    };
    
    /**
     * User management remove
     * @param {String} gid   group id
     */
    Contextboard.prototype.userManagementRemove = function(gid) {
        let inputs = this.place.find(`.contextboard-usertable[data-gid="${gid}"] input.contextboard-users-selectuser:checked`);
        let uids = inputs.map(function() {return $(this).attr('data-user');}).get();
        let request = {
            actions: [],
            appid: this.appid,
            apptype: this.apptype,
            contextid: this.contextid,
            contexttype: this.contexttype
        };
        for (let i = 0, len = uids.length; i < len; i++) {
            request.actions.push({
                actiontype: 'remove',
                grouphash: gid,
                username: uids[i]
            });
        };
        this.startServerWait();
        this.place.trigger('usermanagement', request);
    };
    
    /**
     * User management add
     * @param {String} username   Username for the user
     * @param {String} gid        Group id
     */
    Contextboard.prototype.userManagementAdd = function(username, gid) {
        let request = {
            actions: [{
                actiontype: 'add',
                grouphash: gid,
                username: username
            }],
            appid: this.appid,
            apptype: this.apptype,
            contextid: this.contextid,
            contexttype: this.contexttype
        };
        this.startServerWait();
        this.place.trigger('usermanagement', request);
    };
    
    /**
     * Request updates for contextinfo
     */
    Contextboard.prototype.requestContextinfoUpdate = function() {
        this.place.trigger('makeupdaterequest');
    };
    
    /**
     * Remove selected users from the group (visual list and userlist)
     * @param {String} actions    User management actions
     */
    Contextboard.prototype.modifyUsers = function(actions) {
        actions = actions || [];
        for (let i = 0, len = actions.length; i < len; i++) {
            let action = actions[i];
            switch (action.actiontype) {
                case 'remove':
                    this.users.removeMembers([action.username], [action.grouphash]);
                    if (this.users.getGroupsByUser(action.username).length === 0) {
                        this.users.removeUser(action.username);
                    };
                    let row = this.place.find(`.contextboard-usergroup[data-gid="${escapeHTML(action.grouphash)}"] tr[data-username="${action.username}"]`);
                    row.remove();
                    //this.view();
                    break;
                case 'add':
                    if (!this.users.users[action.username]) {
                        this.users.addUser({username: action.username}, action.grouphash);
                    } else {
                        this.users.addMembers(action.username, action.grouphash);
                    };
                    this.view();
                    // Return the focus to the same input field for further additions.
                    this.lastfocus = {
                        selector: `.contextboard-usergroup[data-gid="${escapeHTML(action.grouphash)}"] input.contextboard-addusername`
                    };
                    break;
                default:
                    break;
            };
        };
    };
    
    /**
     * Start server wait animation
     */
    Contextboard.prototype.startServerWait = function() {
        this.place.append(this.getTemplate('waitbox'));
    };

    /**
     * Stop server wait animation
     * @param {Object} data    Data about the connection
     */
    Contextboard.prototype.stopServerWait = function(data) {
        let uilang = this.settings.uilang;
        this.place.find('.contextboard-waitshadow').remove();
        if (!data.success) {
            var errormessage = '';
            switch (data.errortype) {
                case 5:
                    errormessage = ebooklocalizer.localize('contextboard:incorrect_username', uilang);
                    break;
                case 1:
                    errormessage = ebooklocalizer.localize('contextboard:connectionerror', uilang);
                    break;
                default:
                    errormessage = 'Error type: ' + data.errortype + '\nError message: ' + data.errormessage;
                    break;
            };
            alert(ebooklocalizer.localize('contextboard:sendfailed', uilang) + '\n' + errormessage);
        };
    };
    
    /**
     * Set focus to last focus
     */
    Contextboard.prototype.setFocus = function() {
        if (this.lastfocus && this.lastfocus.selector) {
            this.place.find(this.lastfocus.selector).focus();
            this.lastfocus = false;
        };
    };

    /**
     * Close the tool.
     */
    Contextboard.prototype.close = function() {
        this.place.trigger('closeappok', {appid: this.appid, apptype: this.apptype});
    };

    /**
     * Get html templates
     * @param {String} templateid   The id of the requested template.
     */
    Contextboard.prototype.getTemplate = function(templateid) {
        let uilang = this.settings.uilang;
        let templates = {
            html: `
                <div class="contextboard-contextinfo">
                </div>
                <div class="contextboard-userlist">
                </div>
            `,
            waitbox: `
                <div class="contextboard-waitshadow">
                    <div class="contextboard-waitspinner">
                    ${Contextboard.prototype.icons.waitspinner}<br>
                    <button class="ffwidget-button contextboard-actionbutton" data-action="interrupt_send">${ebooklocalizer.localize('contextboard:interrupt', uilang)}</button>
                    </div>
                </div>
            `
        };
        let result = templates[templateid];
        return result;
    };

    /**
     * Set the css styling.
     */
    Contextboard.prototype.setStyles = function() {
        if ($('head style#contextboardstyles').length === 0) {
            $('head').append('<style type="text/css" id="contextboardstyles">'+this.styles+'</style>')
        };
    };

    /**
     * Defaults for the tool.
     */
    Contextboard.prototype.defaults = {
        settings: {
            uilang: 'en'
        }
    };

    /**
     * The styles for the tool.
     */
    Contextboard.prototype.styles = `
        .contextboard {background-color: #eee; font-family: helvetica, Arial, sans-serif; max-height: 100%; overflow: auto; padding: 1em;}
        .contextboard h1 {text-align: center;}
        .contextboard-usergroup {
            box-shadow: 0 0 2px rgba(0,0,0,0.5);
            border: 1px solid #888;
            margin: 1em auto;
        }
        .contextboard-usertable {border-collapse: collapse; font-size: 80%; width: 100%; background-color: white;}
        .contextboard-usertable th, .contextboard-usertable td {padding: 0.2em 0.5em; text-align: left; vertical-align: top;}
        .contextboard-usertable tbody tr:nth-child(even) {background-color: rgba(0,0,0,0.05);}
        .contextboard-usertable tbody tr:nth-child(odd) {background-color: rgba(255,255,255,0.2);}
        .contextboard-usertable .contextboard-selectcell {width: 1em;}
        .contextboard-usertable tfoot {border-top: 1px solid #888; background-color: rgba(0,0,0,0.1);}
        .contextboard-usertable caption {font-weight: bold; font-size: 120%; padding: 0.2em;}
        .contextboard-infotable {width: 100%;}
        .contextboard-infotable td, .contextboard-infotable th {padding: 0.2em 0.5em; text-align: left; vertical-align: top;}
        .contextboard-subjectlist {list-style: none; margin: 0; padding: 0;}
        .contextboard-subjectlist li {
            display: inline-block;
            margin: 0 0.5em;
            padding: 4px;
            text-align: center;
            border-radius: 4px;
            background-color: rgba(255,255,255,0.3);
            max-width: 50px;
            overflow: hidden;
        }
        .contextboard-subjecticon {line-height: 0;}
        .contextboard-subjecttext {font-family: sans-serif; font-size: 70%;}
        .contextboard-contextinfo {
            border: 1px solid #999;
            background-color: white;
            box-shadow: 0 0 2px rgba(0,0,0,0.5);
        }
        table.contextboard-infotable tr:nth-child(even) {
            background-color: rgba(0,0,0,0.05);
        }
        table.contextboard-infotable th {vertical-align: middle; font-size: 130%;}
        .contextboard-infolabel {font-family: sans-serif;}
        .contextboard-info-school {font-style: italic;}
        .contextboard-messagelink svg {width: 1.3em; height: 1.3em; display: inline-block; vertical-align: bottom; cursor: pointer;}
        .contextboard-waitshadow {
            position: absolute;
            top: 0; bottom: 0; left: 0; right: 0;
            z-index: 300;
            display: flex;
            flex-direction: column;
            justify-content: center;
            align-items: center;
            background-color: rgba(0,0,0,0.2);
        }
        .contextboard-waitshadow .contextboard-waitspinner {text-align: center;}
    `;

    /**
     * Icons
     */
    Contextboard.prototype.icons = {
        appicon: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-contextboard"><path style="stroke: none;" d="M15 9 a5 5 0 0 1 0 10 a5 5 0 0 1 0 -10z m-10 16 a6 6 0 0 1 6 -6 l4 4 l4 -4 a6 6 0 0 1 6 6 v3 h-20z m0 -14 a3 3 0 1 1 2 0 a5 5 0 0 1 2 1 a6 6 0 0 0 1 5 h-9 v-2 a5 5 0 0 1 5 -4z m19 0 a5 5 0 0 1 5 4 v2 h-9 a6 6 0 0 0 1 -5 a5 5 0 0 1 2 -1 a3 3 0 1 1 2 0z" /></svg>',
        refresh: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icon mini-icon-refresh"><path style="stroke: none;" d="M16 14 l8 0 l0 -8 l-3 3 a8.5 8.5 0 1 0 0 12 l-2 -2 a5.6 5.6 0 1 1 0 -8 z"></path></svg>',
        message: '<svg class="mini-icon mini-icon-message" viewBox="0 0 30 30" height="20" width="20" xmlns="http://www.w3.org/2000/svg"><path d="M3 5 l24 0 l0 18 l-24 0z" style="stroke: none; fill: white;"></path> <path d="M3 5 l24 0 l0 18 l-24 0z m1 1 l0 2 l11 8 l11 -8 l0 -2z m0 16 l22 0 l-8 -7 l-3 2 l-3 -2z m0 -1 l7 -6.5 l-7 -5.5z m22 0 l0 -12 -7 5.5z" style="stroke: none;"></path></svg>',
        waitspinner: '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="50" height="50" viewBox="0 0 100 100" preserveAspectRatio="xMidYMid" class="wait-circle-1"><circle style="stroke: rgba(0,0,0,0.2); stroke-width: 10; fill: none;" cx="50" cy="50" r="40"></circle><path class="waitspinner" style="stroke-linecap: round; stroke-width: 10; fill: none;" stroke="rgba(0,0,0,0.5)" d="M50 10 a40 40 0 1 1 -40 40"><animateTransform attributeType="xml" attributeName="transform" type="rotate" from="0 50 50" to="360 50 50" dur="2s" repeatCount="indefinite" /></path></svg>',
    };

    /**
     * Localization
     */
    Contextboard.localization = {
        "en": {
            "contextboard:name": "Name",
            "contextboard:username": "Username",
            "contextboard:email": "Email",
            "contextboard:school": "School",
            "contextboard:teacher": "Teacher",
            "contextboard:shortdescription": "Short description",
            "contextboard:description": "Description",
            "contextboard:starttime": "Starts at",
            "contextboard:endtime": "Ends at",
            "contextboard:add": "Add",
            "contextboard:remove": "Remove",
            "contextboard:interrupt": "Interrupt",
            "contextboard:sendfailed": "Sending failed",
            "contextboard:incorrect_username": "Incorrect username",
            "contextboard:connectionerror": "Connection failed"
        },
        "fi": {
            "contextboard:name": "Nimi",
            "contextboard:username": "Tunnus",
            "contextboard:email": "Sähköposti",
            "contextboard:school": "Koulu",
            "contextboard:teacher": "Opettaja",
            "contextboard:shortdescription": "Lyhyt kuvaus",
            "contextboard:description": "Kuvaus",
            "contextboard:starttime": "Alkaa",
            "contextboard:endtime": "Päättyy",
            "contextboard:add": "Lisää",
            "contextboard:remove": "Poista",
            "contextboard:interrupt": "Keskeytä",
            "contextboard:sendfailed": "Lähetys epäonnistui",
            "contextboard:incorrect_username": "Virheellinen käyttäjätunnus",
            "contextboard:connectionerror": "Yhteys epäonnistui"
        },
        "sv": {
            "contextboard:name": "Namn",
            "contextboard:username": "Användarnamn",
            "contextboard:email": "E-post",
            "contextboard:school": "Skolan",
            "contextboard:teacher": "Läraren",
            "contextboard:shortdescription": "Kort beskrivning",
            "contextboard:description": "Beskrivning",
            "contextboard:starttime": "Börjar",
            "contextboard:endtime": "Slutar",
            "contextboard:add": "Lägg till",
            "contextboard:remove": "Ta bort",
            "contextboard:interrupt": "Avbryta",
            "contextboard:sendfailed": "Sändning misslyckades",
            "contextboard:incorrect_username": "Felaktigt användarnamn",
            "contextboard:connectionerror": "Anslutningen misslyckades"
        }
    }
    
    Contextboard.tourguide = {
        id: 'contextboard',
        autostart: true,
        items: [
            {
                id: 'contextboard',
                selector: '.virumcourse-applistitem[data-apptype="contextboard"]',
                side: 'left',
                position: 'start',
                title: {
                    en: 'Course Information',
                    fi: 'Kurssin tiedot',
                    sv: 'Kurs information'
                },
                message: {
                    en: 'This tool shows all available information about the course. For teachers it shows also the lists of all students and teachers on the course and user interface to add and remove them.',
                    fi: 'Tämä työkalu näyttää saatavilla olevat tiedot kurssista. Opettajille ovat lisäksi näkyvissä listat kurssilla olevista opiskelijoista ja opettajista sekä käyttöliittymä heidän lisäämiseksi ja poistamiseksi.',
                    sv: 'This tool shows all available information about the course. For teachers it shows also the lists of all students and teachers on the course and user interface to add and remove them.'
                },
                enter: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="contextboard"]:not(.tabactive)',
                        event: 'click'
                    }
                ],
                leave: [
                    {
                        selector: '.virumcourse-applistitem[data-apptype="contextboard"].tabactive',
                        event: 'click'
                    }
                ]
            }
        ]
    };

    try {
        ebooklocalizer.addTranslations(Contextboard.localization);
    } catch (err) {
        ebooklocalizer = {
            translations: {},
            addTranslations: function(trans){
                this.translations = $.extend(true, this.translations, trans);
            },
            localize: function(key, lang){
                lang = (this.translations[lang] ? lang : 'en');
                return this.translations[lang] && this.translations[lang][key] || key;
            }
        }
        ebooklocalizer.addTranslations(Contextboard.localization);
    }


    /**** jQuery-plugin *****/
    var methods = {
        'init': function(params){
            return this.each(function(){
                var board = new Contextboard(this, params);
            });
        },
        'getdata': function(){
            var $place = $(this).eq(0);
            $place.trigger('getdata');
            var data = $place.data('[[contextboarddata]]');
            return data;
        },
        'geticon': function() {
            return Contextboard.prototype.icons.appicon;
        },
        'gettitle': function() {
            return '';
        }
    }

    $.fn.contextboard = function(method){
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof(method) === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method ' + method + ' does not exist in coursedashboard.');
            return false;
        }
    };


})(window, jQuery, window.ebooklocalizer);
