/***
 * TourGuide - Tool for giving guided tour on html-page
 * Petri Salmela <petri.salmela@fourferries.fi>
 ***/

/**
 * Requirements:
 * - jQuery
 */

try {
    typeof(jQuery) !== 'undefined';
} catch (err) {
    throw new Error('Missing dependency in ' + err.fileName + '\n' + err);
}

(function ($) {
    { /*** jQuery plugin ***/
        $.fn.tourguide = function(options){
            if (methods[options]){
                return methods[options].apply( this, Array.prototype.slice.call( arguments, 1));
            } else if (typeof(options) === 'object' || !options) {
                return methods.init.apply(this, arguments);
            } else {
                $.error( 'Method ' +  method + ' does not exist on Navbar' );
                return this;
            }
        }
        
        var methods = {
            init: function(params){
                params = $.extend(true, {
                }, params);
                var tour = new TourGuide(this, params);
                window.tour = tour;
            },
            getdata: function(params){
                var $place = $(this);
                $place.trigger('getdata');
                var data = $place.data('[[tourguidedata]]');
                return data;
            }
        }
    }

    /**
     * Helper functions
     */

    /**
     * Sanitizer
     */
    var sanitize = function(text, options) {
        options = $.extend({
            SAFE_FOR_JQUERY: true
        }, options);
        var result;
        if (DOMPurify) {
            result = DOMPurify.sanitize(text, options);
        } else {
            result = escapeHTML(text);
        };
        return result;
    };
    
    /**
     * Escape html for security
     */
    var escapeHTML = function(html) {
        return document.createElement('div')
            .appendChild(document.createTextNode(html))
            .parentNode
            .innerHTML
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
    };

    
    { /*** TourGuide class ***/
        /******
         * Class for Guided Tour
         * @constructor
         * @param {jQuery} place - place for tourguide
         * @param {Object} options - settings for tourguide
         ******/
        var TourGuide = function(place, options){
            this.place = $(place);
            options = $.extend(true, {}, this.defaults, options);
            this.id = options.data.id;
            this.data = options.data;
            this.settings = options.settings;
            this.initData();
            this.initHandlers();
            this.selectTour('all');
            this.active = false;
            this.show();
            window.tourguide = this;
        }
        
        /******
         * Default settings for tourguide
         ******/
        TourGuide.prototype.defaults = {
            data: {
                id: 'dummydefaulttour',
                tours: {
                },
                "static": false
            },
            settings: {
                uilang: 'en'
            }
        }
        
        /******
         * Init tourguide data.
         ******/
        TourGuide.prototype.initData = function(){
            this.place.addClass('tourguide-wrapper');
            this.current = null;
            this.currentTour = null;
            this.currentTourId = '';
            this.tours = Object.keys(this.data.tours);
            this.generateById();
        };
        
        /**
         * Generate itemsById map
         */
        TourGuide.prototype.generateById = function() {
            this.allCounter = 0;
            this.autoCounter = 0;
            this.autostart = [];
            this.itemsById = {all: {}, autostart: {}};
            for (let i = 0, len = this.tours.length; i < len; i++) {
                let tourid = this.tours[i];
                this.itemsById[tourid] = {};
                let tour = this.data.tours[tourid];
                if (tour.autostart) {
                    this.autostart.push(tour.id);
                };
                for (let j = 0, tourlen = tour.items.length; j < tourlen; j++) {
                    let item = tour.items[j];
                    this.itemsById[tourid][item.id] = {index: j, item: item};
                    this.itemsById['all'][tourid + ':' + item.id] = {index: this.allCounter, item: item};
                    this.allCounter++;
                    if (tour.autostart) {
                        this.itemsById['autostart'][tourid + ':' + item.id] = {index: this.autoCounter, item: item};
                        this.autoCounter++;
                    };
                };
            };
        };
        
        /**
         * Add tour to the TourGuide
         * @param {Object} tour    One tour in the same format as in initing object
         */
        TourGuide.prototype.addTour = function(tour) {
            var tourid = tour.id;
            var oldTour = this.data.tours[tourid];
            this.data.tours[tourid] = tour;
            if (oldTour) {
                // Existing tour was updated. Generate whole itemsById from scratch.
                this.generateById();
            } else {
                // This was new tour. Just add new items to itemById.
                this.tours.push(tourid);
                this.itemsById[tourid] = {};
                if (tour.autostart) {
                    this.autostart.push(tourid);
                };
                for (let i = 0, len = tour.items.length; i < len; i++) {
                    let item = tour.items[i];
                    this.itemsById[tourid][item.id] = {index: i, item: item};
                    this.itemsById['all'][tour + ':' + item.id] = {index: this.allCounter, item: item};
                    this.allCounter++;
                    if (tour.autostart) {
                        this.itemsById['autostart'][tour + ':' + item.id] = {index: this.autoCounter, item: item};
                        this.autoCounter++;
                    };
                };
            };
        };
        
        /**
         * Select the current tour
         * @param {String} tourid    The id of the tour to be selected.
         * @returns {String|Boolean}    The selected tourid or false
         */
        TourGuide.prototype.selectTour = function(tourid) {
            var tour = this.data.tours[tourid];
            var selected = false;
            if (tour) {
                // Use selected tour as currentTour
                this.currentTourId = tourid;
                this.currentTour = tour;
                selected = tourid;
            } else if (tourid === 'autostart') {
                // Concat all tours to be 'all'-tour.
                this.currentTourId = 'autostart';
                this.currentTour = {
                    id: 'autostart',
                    autostart: true,
                    items: [],
                    afterbuttons: []
                };
                for (let i = 0, len = this.autostart.length; i < len; i++) {
                    let itour = this.data.tours[this.autostart[i]];
                    this.currentTour.items = this.currentTour.items.concat(itour.items);
                    if (itour.afterbuttons && itour.afterbuttons.length > 0) {
                        this.currentTour.afterbuttons = this.currentTour.afterbuttons.concat(itour.afterbuttons);
                    };
                };
                selected = tourid;
            } else if (tourid === 'all') {
                // Concat all tours to be 'all'-tour.
                this.currentTourId = 'all';
                this.currentTour = {
                    id: 'all',
                    autostart: true,
                    items: []
                };
                for (let i = 0, len = this.tours.length; i < len; i++) {
                    this.currentTour.items = this.currentTour.items.concat(this.data.tours[this.tours[i]].items)
                };
                selected = tourid;
            };
            this.setNavidots();
            this.setAfters();
            this.setAutostart();
            return selected;
        };
        
        /**
         * Draw tourguide
         */
        TourGuide.prototype.show = function() {
            this.addStyles();
            this.place.html(this.data.static ? this.templates.static : this.templates.html);
            this.infoarea = this.place.find('.tourguide-infoarea');
            this.navLabel = this.place.find('.tourguide-nav-label');
            this.labelbox = this.place.find('.tourguide-labelbox');
            this.progressbar = this.place.find('.tourguide-progressbar');
            this.navidots = this.place.find('.tourguide-navidots');
            this.afterarea = this.place.find('.tourguide-afterarea');
            this.autostartarea = this.place.find('.tourguide-autostartarea');
        };
        
        /**
         * Set navidots
         */
        TourGuide.prototype.setNavidots = function() {
            if (this.navidots && this.currentTour) {
                this.navidots.empty();
                for (let i = 0, len = this.currentTour.items.length; i < len; i++) {
                    this.navidots.append(`<div class="tourguide-navidots-item" data-navi-index="${i}" title="${i+1}"></div>`);
                };
            };
        };
        
        /**
         * Set after buttons of selected tour
         */
        TourGuide.prototype.setAfters = function() {
            var uilang = this.settings.uilang;
            if (this.afterarea && this.afterarea.length > 0 && this.currentTour) {
                this.afterarea.empty();
                var buttons = this.currentTour.afterbuttons || [];
                var buttemplate = this.templates.button;
                for (let i = 0, len = buttons.length; i < len; i++) {
                    let butt = buttons[i];
                    let button = $(buttemplate);
                    button.attr('data-index', i);
                    let title = (typeof(butt.title) === 'string' ? butt.title : (butt.title && (butt.title[uilang] || butt.title['en'])) || '');
                    button.attr('title', escapeHTML(title));
                    let label = (typeof(butt.label) === 'string' ? butt.label : (butt.label && (butt.label[uilang] || butt.label['en'])) || '');
                    button.text(escapeHTML(label));
                    this.afterarea.append(button);
                };
            };
        };
        
        /**
         * Set area for autostart options.
         */
        TourGuide.prototype.setAutostart = function() {
            let uilang = this.settings.uilang;
            if (this.autostartarea && this.autostartarea.length > 0 && this.currentTour) {
                this.autostartarea.empty();
                if (this.currentTour.autostart) {
                    let dontshow = this.localizations[uilang] && this.localizations[uilang]['dontshow_autostart'] || this.localizations['en']['dontshow_autostart'];
                    let state = this.getConditionalState('autostart');
                    let html = `
                        <div class="tourguide-dontshow">
                            <label><input type="checkbox" class="tourguide-dontshowtick"> <span>${dontshow}</span></label>
                        </div>`;
                    this.autostartarea.append(html);
                    this.autostartarea.find('.tourguide-dontshowtick').prop('checked', state);
                };
            };
        };
        
        /**
         * Init handlers
         */
        TourGuide.prototype.initHandlers = function() {
            var tool = this;
            this.place.off('activate').on('activate', function(event, data) {
                event.stopPropagation();
                tool.activate(data);
            }).off('deactivate').on('deactivate', function(event, data) {
                event.stopPropagation();
                tool.deactivate();
            }).off('toggleactivate').on('toggleactivate', function(event, data) {
                event.stopPropagation();
                tool.toggleActivate(data);
            }).off('conditionalactivate').on('conditionalactivate', function(event, data) {
                event.stopPropagation();
                tool.conditionalActivate(data);
            }).off('autostart').on('autostart', function(event) {
                event.stopPropagation();
                tool.doAutostart();
            }).off('click', '.tourguide-close').on('click', '.tourguide-close', function(event, data) {
                event.stopPropagation();
                tool.deactivate();
            }).off('click', '.tourguide-nav-previous').on('click', '.tourguide-nav-previous', function(event, data) {
                event.stopPropagation();
                tool.showPrevItem();
            }).off('click', '.tourguide-nav-next').on('click', '.tourguide-nav-next', function(event, data) {
                event.stopPropagation();
                tool.showNextItem();
            }).off('click', '.tourguide-progressarea').on('click', '.tourguide-progressarea', function(event, data) {
                event.stopPropagation();
                tool.moveProgress(event.offsetX);
            }).off('click', '.tourguide-navidots-item').on('click', '.tourguide-navidots-item', function(event, data) {
                event.stopPropagation();
                var index = $(this).attr('data-navi-index') | 0;
                tool.showItem(index);
            }).off('click', '.tourguide-afterbutton').on('click', '.tourguide-afterbutton', function(event, data) {
                event.stopPropagation();
                var index = $(this).attr('data-index');
                tool.afterAction(index);
            }).off('addtour').on('addtour', function(event, tour) {
                event.stopPropagation();
                tool.addTour(tour);
            }).off('change', 'input.tourguide-dontshowtick').on('change', 'input.tourguide-dontshowtick', function(event) {
                event.stopPropagation();
                let checked = $(this).is(':checked');
                tool.setConditionalState({tourid: 'autostart', state: checked});
            });
            $(window).off('resize.tourguide').on('resize.tourguide', function(event, tour) {
                event.stopPropagation();
                tool.setLabelPosition();
            });
        };
        
        /**
         * Leave from current item. Trigger events that are required, when leaving a tour item
         */
        TourGuide.prototype.leaveItem = function() {
            // If the id was correct id or index.
            if (this.current && this.current.item && this.current.item.leave) {
                let leave = this.current.item.leave;
                // Trigger all leave events with corresponding optional data.
                for (let i = 0, len = leave.length || 0; i < len; i++) {
                    if (leave[i].delay) {
                        setTimeout(function(){
                            let oldtarget = $(leave[i].selector || '').filter(':visible').eq(0);
                            oldtarget.trigger(leave[i].event || '', leave[i].data || []);
                        }, leave[i].delay);
                    } else {
                        let oldtarget = $(leave[i].selector || '').filter(':visible').eq(0);
                        oldtarget.trigger(leave[i].event || '', leave[i].data || []);
                    };
                };
            };
        };
        
        /**
         * Enter to current item. Trigger events that are required, when entering a tour item.
         */
        TourGuide.prototype.enterItem = function() {
            if (this.current && this.current.item && this.current.item.enter) {
                let enter = this.current.item.enter;
                // Trigger all entering events with corresponding optional data.
                for (let i = 0, len = enter.length || 0; i < len; i++) {
                    if (enter[i].delay) {
                        setTimeout(function(){
                            let newtarget = $(enter[i].selector || '').filter(':visible').eq(0);
                            newtarget.trigger(enter[i].event || '', enter[i].data || []);
                        }, enter[i].delay);
                    } else {
                        let newtarget = $(enter[i].selector || '').filter(':visible').eq(0);
                        newtarget.trigger(enter[i].event || '', enter[i].data || []);
                    };
                };
            };
        };
        
        /**
         * Show tour item
         * @param {Number|String} id   Index or the key for the item to show
         * @param {String} [direction]  An optional direction we are going. '+' === forward, '-' === backwards. Defaults for '+'
         */
        TourGuide.prototype.showItem = function(id, direction) {
            var self = this;
            direction = direction || '+';
            var uilang = this.settings.uilang;
            var item = this.getItem(id);
            if (item) {
                this.leaveItem();
                this.current = item;
                let html = [];
                html.push('<div class="tourguide-touritemtitle">' + sanitize(item.item.title && (item.item.title[uilang] || item.item.title['en']) || '') + '</div>');
                html.push('<div class="tourguide-touritemmessage">' + sanitize(item.item.message[uilang] || item.item.message['en'] || '') + '</div>');
                this.infoarea.html(html.join('\n'));
                this.navLabel.html(item.index + 1);
                if (this.data.static) {
                    this.labelbox.text(item.index + 1);
                };
                this.enterItem();
                setTimeout(function(){self.setLabelPosition();}, 100);
                setTimeout(function(){self.setLabelPosition();}, 500);
                this.progressbar.css({width: (item.index / (this.currentTour.items.length - 1)) * 100 + '%'});
                this.navidots.children().removeClass('dotselected').filter(`[data-navi-index="${item.index}"]`).addClass('dotselected');
                let isfirst = (item.index === 0);
                let islast = (item.index === this.currentTour.items.length - 1);
                this.place.attr('data-isfirst', isfirst + '').attr('data-islast', islast + '');
            };
        };
        
        /**
         * Show the next tour item
         */
        TourGuide.prototype.showNextItem = function() {
            if (this.current && this.current.index >= 0 && this.current.index < this.currentTour.items.length - 1) {
                var index = Math.min((this.current && this.current.index || 0) + 1, this.currentTour.items.length -1);
                this.showItem(index, '+');
            };
        };
        
        /**
         * Show the previous tour item
         */
        TourGuide.prototype.showPrevItem = function() {
            if (this.current && this.current.index > 0 && this.current.index < this.currentTour.items.length) {
                var index = Math.max((this.current && this.current.index || 0) - 1, 0);
                this.showItem(index, '-');
            };
        };
        
        /**
         * Move by clicking on progressbar
         * @param {Number} xpos   x-osition of the mouse on the document
         */
        TourGuide.prototype.moveProgress = function(xpos) {
            var ratio = xpos / this.place.find('.tourguide-progressarea').width();
            var index = Math.round(ratio * (this.currentTour.items.length - 1));
            this.showItem(index);
        };
        
        
        /**
         * Get the tour item
         * @param {String|Number} id   The id or the index of the item
         * @returns {Object}           An object of format: {index: <index of item>, item: <item itself>}
         */
        TourGuide.prototype.getItem = function(id) {
            var item;
            if (typeof(id) === 'number' && this.currentTour.items[id]) {
                item = {index: id, item: this.currentTour.items[id]};
            } else if (typeof(id) === 'string' && this.itemsById[this.currentTourId]) {
                let tourById = this.itemsById[this.currentTourId];
                if (tourById) {
                    item = tourById[id] || {index: 0, item: this.currentTour.items[0]};
                } else {
                    item = {index: 0, item: this.currentTour.items[0]};
                };
            };
            return item;
        };
        
        /**
         * Set the place of the label
         */
        TourGuide.prototype.setLabelPosition = function() {
            var item = this.current && this.current.item;
            var osparent = this.labelbox.offsetParent();
            var poffset = osparent.offset();
            if (item) {
                var position = item.position || 'start';
                var wwidth = $(window).width();
                var wheight = $(window).height();
                var target = $(item.selector || '').filter(':visible').eq(0);
                var tpos = target.offset();
                var tsize = {width: target.outerWidth(), height: target.outerHeight()};
                var lsize = {width: this.labelbox.outerWidth(), height: this.labelbox.outerHeight()};
                // By default, position the label in the middle of the window.
                var style = {top: ($(window).height() - lsize.height)/2, left: ($(window).width() - lsize.width)/2};
                if (target.length > 0) {
                    if (item.side === 'left' && lsize.width + 20 > tpos.left) {
                        item.side = 'right';
                    } else if (item.side === 'right' && (tpos.left + tsize.width + 20 + lsize.width > wwidth)) {
                        item.side = 'left';
                    };
                    switch (item.side) {
                        // Decide the position by the target and the side.
                        case 'left':
                            style = {
                                top: Math.max(0, position === 'end' ? tpos.top +(tsize.height/2) - lsize.height +15 : (position === 'center' ? tpos.top + (tsize.height / 2) - (lsize.height / 2) : tpos.top + (tsize.height/2) - 38)) - poffset.top,
                                left: Math.max(0, tpos.left - lsize.width - 22) - poffset.left
                            };
                            break;
                        case 'right':
                            style = {
                                top: Math.max(0, position === 'end' ? tpos.top +(tsize.height/2) - lsize.height +15 : (position === 'center' ? tpos.top + (tsize.height / 2) - (lsize.height / 2) : tpos.top + (tsize.height/2) - 38)) - poffset.top,
                                left: Math.max(0, tpos.left + tsize.width + 22) - poffset.left
                            };
                            break;
                        case 'up':
                            if (tpos.left + lsize.width > wwidth) {
                                position = 'end';
                            };
                            style = {
                                top: Math.max(0, tpos.top - lsize.height - 18) - poffset.top,
                                left: Math.max(0, position === 'end' ? tpos.left + tsize.width/2 - lsize.width + 35 : (position === 'center' ? tpos.left + tsize.width / 2 -lsize.width / 2 : tpos.left + tsize.width/2 - 35)) - poffset.left
                            };
                            break;
                        case 'down':
                            if (tpos.left + lsize.width > wwidth) {
                                position = 'end';
                            };
                            style = {
                                top: Math.max(0, tpos.top + tsize.height + 18) - poffset.top,
                                left: Math.max(0, position === 'end' ? tpos.left + tsize.width/2 - lsize.width + 35 : (position === 'center' ? tpos.left + tsize.width / 2 -lsize.width / 2 : tpos.left + tsize.width/2 - 35)) - poffset.left
                            };
                            break;
                        case 'inside':
                            let pos = position.split(' ');
                            style = {
                                top: Math.max(0, pos[0] === 'start' ? tpos.top : pos[0] === 'center' ? tpos.top + (tsize.height/2) - (lsize.height/2) : tpos.top + tsize.height - lsize.height) - poffset.top,
                                left: Math.max(0, pos[1] === 'start' ? tpos.left : pos[1] === 'center' ? tpos.left + (tsize.width/2) -(lsize.width/2) : tpos.left + tsize.width - lsize.width) - poffset.left
                            };
                            break;
                        default:
                            style = {};
                            break;
                    };
                    this.labelbox.css(style).attr('data-side', item.side).attr('data-position', position);
                } else {
                    this.labelbox.css(style).attr('data-side', 'none');
                };
            };
        };
        
        /**
         * Do after action on button click
         * @param {Number} index   The index of the clicked afterbutton
         */
        TourGuide.prototype.afterAction = function(index) {
            var button = this.currentTour.afterbuttons[index];
            var selector = escapeHTML(button.selector);
            var event = button.event;
            var data = button.data;
            $(selector).trigger(event, data);
        };
        
        /**
         * Activate tourguide
         * @param {Object} data              Data for activation
         * @param {String} data.tour         The id of the tour to activate
         * @param {String|Number} data.item  The id or index of the item on the tour
         * @param {Boolean} data.conditional Should this be considered as conditional show. (Optional. default: false)
         */
        TourGuide.prototype.activate = function(data) {
            data = data || {tour: 'all', item: 0};
            data.tour = data.tour || 'all';
            data.item = data.item || 0;
            let selected = this.selectTour(data.tour);
            if (data && selected && typeof(data.item) !== 'undefined') {
                this.place.addClass('tourguide-active');
                this.active = true;
                this.showItem(data.item);
            };
        };
        
        /**
         * Deactivate tourguide
         * @param {Object} data  Data for deactivation
         */
        TourGuide.prototype.deactivate = function(data) {
            this.leaveItem();
            this.place.removeClass('tourguide-active');
            this.active = false;
        };
        
        /**
         * Toggle active state
         * @param {Object} data  Data for activation/deactivation.
         */
        TourGuide.prototype.toggleActivate = function(data) {
            if (this.active) {
                this.deactivate(data);
            } else {
                this.activate(data);
            };
        };

        /**
         * Conditional activate
         * Activate tour, if the user has not marked it as "don't show again".
         * @param {Object} data               Data for activation
         * @param {String} data.tour          The id of the tour to activate
         * @param {String|Number} data.item   The id or index of the item on the tour
         */
        TourGuide.prototype.conditionalActivate = function(data) {
            data = data || {};
            let tour = data.tour;
            let saved = this.loadLocal(tour);
            if (!saved.conditionaldontshow) {
                this.activate(data);
            };
        };
        
        /**
         * Start the autostart tour conditionally.
         */
        TourGuide.prototype.doAutostart = function() {
            this.conditionalActivate({tour: 'autostart', item: 0});
        };
        
        /**
         * Get state of conditional showing for current tour
         * @param {String} tourid   The id of the tour (optional)
         * @returns {Boolean}       Whether the conditional showing is denied or not (true: "don't show", false: "show")
         */
        TourGuide.prototype.getConditionalState = function(tourid) {
            tourid = tourid || this.currentTourId;
            let data = this.loadLocal(tourid);
            return data.conditionaldontshow || false;
        };
        
        /**
         * Set state of conditional showing for current tour
         * @param {Object} sdata          The data for setting conditional state
         * @param {String} sdata.tourid   The tour that the conditional state is set
         * @param {Boolean} sdata.state   The state of "dontshow" (true: "don't show", false: "show")
         */
        TourGuide.prototype.setConditionalState = function(sdata) {
            let tourid = sdata.tourid || this.currentTourId;
            let data = this.loadLocal(tourid);
            data.conditionaldontshow = sdata.state;
            this.saveLocal(tourid, data);
        };
        
        /**
         * Load data from localStorage
         * @param {String} key     Key for data
         * @return {Object}        Saved data for given key
         */
        TourGuide.prototype.loadLocal = function(key) {
            let saved = localStorage.getItem(this.id + '_tourguidesaved') || '{}';
            let sdata = {};
            try {
                sdata = JSON.parse(saved);
            } catch (err) {
                sdata = {};
            };
            return sdata[key] || {};
        };
        
        /**
         * Save data to localStorage
         * @param {String} key      Key for data
         * @param {Object} data     Data to save
         */
        TourGuide.prototype.saveLocal = function(key, data) {
            let saved = localStorage.getItem(this.id + '_tourguidesaved') || '{}';
            let sdata = {};
            try {
                sdata = JSON.parse(saved);
            } catch (err) {
                sdata = {};
            };
            sdata[key] = data;
            saved = JSON.stringify(sdata);
            localStorage.setItem(this.id + '_tourguidesaved', saved);
        };
        
        /******
         * Add styles for tourguide
         ******/
        TourGuide.prototype.addStyles = function(){
            if ($('head style#tourguidestyle').length > 0){
                $('head style#tourguidestyle').remove();
            };
            $('head').append(this.getStyles());
        };
        
        /**
         * Localizations
         */
        TourGuide.prototype.localizations = {
            en: {
                "dontshow_autostart": "Don't show automatic guided tour."
            },
            fi: {
                "dontshow_autostart": "Älä näytä automaattista opaskierrosta."
            },
            sv: {
                "dontshow_autostart": "Visa inte automatisk guidad tur."
            }
        }
        
        /**
         * The templates
         */
        TourGuide.prototype.templates = {
            "static": `
                <div class="tourguide-tourarea">
                    <div class="tourguide-nav">
                        <button class="tourguide-nav-previous">
                            <svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icona mini-icon-navleft">
                                <path style="stroke: none;" d="M20 2 v26 l-10 -13z"></path>
                            </svg>
                        </button>
                        <button class="tourguide-nav-label">0</button>
                        <button class="tourguide-nav-next">
                            <svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icona mini-icon-navright">
                                <path style="stroke: none;" d="M10 2 l10 13 l-10 13z"></path>
                            </svg>
                        </button>
                    </div>
                    <div class="tourguide-contentarea">
                        <div class="tourguide-progressarea">
                            <div class="tourguide-progressbar" style="width: 50%;"></div>
                        </div>
                        <div class="tourguide-navidots">
                        </div>
                        <div class="tourguide-infoarea"></div>
                    </div>
                    <div class="tourguide-controlarea">
                        <button class="tourguide-close">&times;</button>
                    </div>
                </div>
                <div class="tourguide-labelarea">
                    <div class="tourguide-label tourguide-labelbox"></div>
                </div>
            `,
            html: `
                <div class="tourguide-tourarea tourguide-biglabel tourguide-labelbox">
                    <div class="tourguide-head">
                        <div class="tourguide-nav">
                            <button class="tourguide-nav-previous">
                                <svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icona mini-icon-navleft">
                                    <path style="stroke: none;" d="M20 2 v26 l-10 -13z"></path>
                                </svg>
                            </button>
                            <button class="tourguide-nav-label">0</button>
                            <button class="tourguide-nav-next">
                                <svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="30" height="30" viewBox="0 0 30 30" class="mini-icona mini-icon-navright">
                                    <path style="stroke: none;" d="M10 2 l10 13 l-10 13z"></path>
                                </svg>
                            </button>
                        </div>
                        <div class="tourguide-navidots">
                        </div>
                        <div class="tourguide-controlarea">
                            <button class="tourguide-close">&times;</button>
                        </div>
                    </div>
                    <div class="tourguide-contentarea">
                        <div class="tourguide-progressarea">
                            <div class="tourguide-progressbar" style="width: 50%;"></div>
                        </div>
                        <div class="tourguide-infoarea"></div>
                    </div>
                    <div class="tourguide-afterarea">
                    </div>
                    <div class="tourguide-autostartarea">
                    </div>
                </div>
            `,
            label: `<div class="tourguide-label">{{{label}}}</div>`,
            button: `<button class="tourguide-afterbutton"></div>`
        };

        /**
         * The css styles for tourguide
         */
        TourGuide.prototype.getStyles = function() {
            var styles = `
                <style id="tourguidestyle" type="text/css">
                    .tourguide-wrapper {display: none;}
                    .tourguide-wrapper.tourguide-active {display: block;}
                    .tourguide-tourarea {
                        display: flex;
                        flex-direction: row;
                        justify-content: space-between;
                        align-items: stretch;
                        background-color: #444;
                        color: #eee;
                        height: 4em;
                    }
                    .tourguide-nav {
                        display: flex;
                        flex-direction: row;
                        justify-content: space-between;
                        align-items: stretch;
                    }
                    .tourguide-nav-label,
                    .tourguide-nav-previous,
                    .tourguide-nav-next,
                    .tourguide-close {
                        color: #009EE3;
                        padding: 0;
                        border: none;
                        background-color: #444;
                        cursor: pointer;
                    }
                    .tourguide-nav-label:hover,
                    .tourguide-nav-previous:hover,
                    .tourguide-nav-next:hover,
                    .tourguide-close:hover {
                        background-color: #555;
                    }
                    .tourguide-nav-label:active,
                    .tourguide-nav-previous:active,
                    .tourguide-nav-next:active,
                    .tourguide-close:active {
                        background-color: #666;
                    }
                    .tourguide-nav-previous svg path,
                    .tourguide-nav-next svg path {
                        fill: #009EE3;
                    }
                    .tourguide-nav-label {
                        font-size: 200%;
                        font-weight: bold;
                        min-width: 1em;
                        color: #009EE3;
                        padding: 0;
                        padding-bottom: 0.2em;
                    }
                    .tourguide-close {
                        font-size: 200%;
                        font-weight: bold;
                        min-width: 1em;
                        color: #009EE3;
                        padding: 0;
                        height: 100%;
                    }
                    .tourguide-contentarea {
                        flex-grow: 1;
                        display: flex;
                        flex-direction: column;
                    }
                    .tourguide-progressarea {
                        background-color: #222;
                        border-left: 20px solid #009EE3;
                        height: 3px;
                        cursor: pointer;
                        z-index: 10;
                        flex-shrink: 0;
                    }
                    .tourguide-progressbar {
                        background-color: #009EE3;
                        height: 100%;
                        margin: 0;
                        padding: 0;
                    }
                    .tourguide-progressarea:hover {
                        height: 10px;
                        margin-bottom: -7px;
                    }
                    .tourguide-navidots {
                        display: flex;
                        flex-direction: row;
                        justify-content: center;
                        margin: 5px;
                        flex-shrink: 0;
                        flex-wrap: wrap;
                    }
                    .tourguide-navidots-item {
                        width: 6px;
                        height: 6px;
                        border: 1px solid #eee;
                        border-radius: 4px;
                        background-color: transparent;
                        flex-grow: 0;
                        margin: 0 5px;
                        cursor: pointer;
                    }
                    .tourguide-navidots-item:hover {
                        box-shadow: 0 0 1px 2px #009EE3;
                    }
                    .tourguide-navidots-item.dotselected {
                        background-color: #009EE3;
                        border-color: #009EE3;
                    }
                    .tourguide-infoarea {
                        flex-grow: 1;
                        padding: 0.1em 1em;
                        overflow: auto;
                        color: #eee;
                        font-family: sans-serif;
                    }
                    .tourguide-touritemtitle {
                        display: block;
                        font-weight: bold;
                        margin-bottom: 0.3em;
                    }
                    .tourguide-biglabel {
                        position: absolute;
                        top: -400px;
                        left: 30%;
                        background-color: #009EE3;
                        color: #eee;
                        font-family: sans-serif;
                        padding: 0.2em;
                        max-width: 30em;
                        min-width: 20em;
                        z-index: 10000;
                        box-shadow: 0 0 0 2px #a00, 0 0 1px 3px white, 20px 20px 100px -25px rgba(0,0,0,0.5);
                        transition: top 0.5s, left 0.5s;
                    }
                    .tourguide-label {
                        min-width: 1em;
                        min-height: 1em;
                        position: fixed;
                        background-color: #009EE3;
                        color: #eee;
                        font-family: sans-serif;
                        font-size: 200%;
                        font-weight: bold;
                        text-align: center;
                        line-height: 100%;
                        padding: 0.2em;
                        z-index: 10000;
                        box-shadow: 0 0 0 2px #444, 5px 5px 8px rgba(0,0,0,0.5);
                        transition: top 0.5s, left 0.5s;
                    }
                    .tourguide-labelbox::after {
                        border: 18px solid #ffb;
                        background-color: transparent;
                        content: "";
                        width: 0;
                        height: 0;
                        position: absolute;
                    }
                    .tourguide-labelbox::before {
                        border: 20px solid #a00;
                        background-color: transparent;
                        content: "";
                        width: 0;
                        height: 0;
                        position: absolute;
                    }
                    .tourguide-labelbox[data-side="none"]::before,
                    .tourguide-labelbox[data-side="none"]::after,
                    .tourguide-labelbox[data-side="inside"]::before,
                    .tourguide-labelbox[data-side="inside"]::after {
                        display: none;
                    }
                    .tourguide-labelbox[data-side="left"]::after {
                        border-right: none;
                        border-top-color: transparent;
                        border-bottom-color: transparent;
                        right: -18px;
                    }
                    .tourguide-labelbox[data-side="left"]::before {
                        border-right: none;
                        border-top-color: transparent;
                        border-bottom-color: transparent;
                        right: -20px;
                    }
                    .tourguide-labelbox[data-side="right"]::after {
                        border-left: none;
                        border-top-color: transparent;
                        border-bottom-color: transparent;
                        left: -18px;
                    }
                    .tourguide-labelbox[data-side="right"]::before {
                        border-left: none;
                        border-top-color: transparent;
                        border-bottom-color: transparent;
                        left: -20px;
                    }
                    .tourguide-labelbox[data-side="up"]::after {
                        border-bottom: none;
                        border-left-color: transparent;
                        border-right-color: transparent;
                        bottom: -18px;
                    }
                    .tourguide-labelbox[data-side="up"]::before {
                        border-bottom: none;
                        border-left-color: transparent;
                        border-right-color: transparent;
                        bottom: -20px;
                    }
                    .tourguide-labelbox[data-side="down"]::after {
                        border-top: none;
                        border-left-color: transparent;
                        border-right-color: transparent;
                        top: -18px;
                    }
                    .tourguide-labelbox[data-side="down"]::before {
                        border-top: none;
                        border-left-color: transparent;
                        border-right-color: transparent;
                        top: -20px;
                    }
                    .tourguide-labelbox[data-side="left"][data-position="start"]::after,
                    .tourguide-labelbox[data-side="right"][data-position="start"]::after {
                        top: 18px;
                    }
                    .tourguide-labelbox[data-side="left"][data-position="center"]::after,
                    .tourguide-labelbox[data-side="right"][data-position="center"]::after {
                        top: 50%;
                        margin-top: -18px;
                    }
                    .tourguide-labelbox[data-side="left"][data-position="end"]::after,
                    .tourguide-labelbox[data-side="right"][data-position="end"]::after {
                        bottom: 18px;
                        top: auto;
                    }
                    .tourguide-labelbox[data-side="up"][data-position="start"]::after,
                    .tourguide-labelbox[data-side="down"][data-position="start"]::after {
                        left: 18px;
                        right: auto;
                    }
                    .tourguide-labelbox[data-side="up"][data-position="center"]::after,
                    .tourguide-labelbox[data-side="down"][data-position="center"]::after {
                        left: 50%;
                        margin-left: -18px;
                    }
                    .tourguide-labelbox[data-side="up"][data-position="end"]::after,
                    .tourguide-labelbox[data-side="down"][data-position="end"]::after {
                        right: 18px;
                        left: auto;
                    }
                    .tourguide-labelbox[data-side="left"][data-position="start"]::before,
                    .tourguide-labelbox[data-side="right"][data-position="start"]::before {
                        top: 16px;
                    }
                    .tourguide-labelbox[data-side="left"][data-position="center"]::before,
                    .tourguide-labelbox[data-side="right"][data-position="center"]::before {
                        top: 50%;
                        margin-top: -20px;
                    }
                    .tourguide-labelbox[data-side="left"][data-position="end"]::before,
                    .tourguide-labelbox[data-side="right"][data-position="end"]::before {
                        bottom: 16px;
                        top: auto;
                    }
                    .tourguide-labelbox[data-side="up"][data-position="start"]::before,
                    .tourguide-labelbox[data-side="down"][data-position="start"]::before {
                        left: 16px;
                        right: auto;
                    }
                    .tourguide-labelbox[data-side="up"][data-position="center"]::before,
                    .tourguide-labelbox[data-side="down"][data-position="center"]::before {
                        left: 50%;
                        margin-left: -20px;
                    }
                    .tourguide-labelbox[data-side="up"][data-position="end"]::before,
                    .tourguide-labelbox[data-side="down"][data-position="end"]::before {
                        right: 16px;
                        left: auto;
                    }
                    .tourguide-biglabel.tourguide-tourarea {
                        height: auto;
                        max-height: 16em;
                        align-items: stretch;
                        background-color: #ffb;
                        color: black;
                        border-radius: 6px;
                        flex-direction: column;
                    }
                    .tourguide-biglabel.tourguide-tourarea .tourguide-head {
                        display: flex;
                        flex-direction: row;
                        justify-content: space-between;
                        align-items: flex-start;
                    }
                    .tourguide-biglabel .tourguide-contentarea {
                        overflow: hidden;
                    }
                    .tourguide-biglabel.tourguide-tourarea .tourguide-infoarea {
                        color: black;
                        padding: 0.5em 1em;
                    }
                    .tourguide-biglabel .tourguide-navidots {
                        flex-shrink: 1;
                        flex-grow: 1;
                    }
                    .tourguide-biglabel .tourguide-navidots-item {
                        border: 1px solid black;
                        box-shadow: inset 1px 1px 1px rgba(0,0,0,0.5);
                        background-color: white;
                        margin: 3px;
                    }
                    .tourguide-biglabel .tourguide-navidots-item.dotselected {
                        background-color: #a00;
                        border-color: #a00;
                    }
                    .tourguide-biglabel .tourguide-navidots-item:hover {
                        box-shadow: inset 1px 1px 1px rgba(0,0,0,0.5), 0 0 1px 2px #a00;
                    }
                    .tourguide-biglabel .tourguide-progressarea {
                        border-left: 20px solid #a00;
                    }
                    .tourguide-biglabel .tourguide-progressbar {
                        background-color: #a00;
                    }
                    .tourguide-biglabel .tourguide-nav-label,
                    .tourguide-biglabel .tourguide-nav-previous,
                    .tourguide-biglabel .tourguide-nav-next,
                    .tourguide-biglabel .tourguide-close {
                        color: #222;
                        background-color: transparent;
                    }
                    .tourguide-biglabel .tourguide-nav-label:hover,
                    .tourguide-biglabel .tourguide-nav-previous:hover,
                    .tourguide-biglabel .tourguide-nav-next:hover,
                    .tourguide-biglabel .tourguide-close:hover {
                        background-color: rgba(255,255,255,0.5);
                    }
                    .tourguide-biglabel .tourguide-nav-previous svg path,
                    .tourguide-biglabel .tourguide-nav-next svg path {
                        fill: black;
                    }
                    .tourguide-wrapper[data-isfirst="true"] .tourguide-biglabel .tourguide-nav-previous,
                    .tourguide-wrapper[data-islast="true"] .tourguide-biglabel .tourguide-nav-next {
                        color: rgba(200,200,200,0.5);
                    }
                    .tourguide-wrapper[data-isfirst="true"] .tourguide-biglabel .tourguide-nav-previous svg path,
                    .tourguide-wrapper[data-islast="true"] .tourguide-biglabel .tourguide-nav-next svg path {
                        fill: rgba(200,200,200,0.5);
                    }
                    /*
                    .tourguide-biglabel.tourguide-labelbox[data-side="left"]::after {
                        border-left: 10px solid #a00;
                    }
                    .tourguide-biglabel.tourguide-labelbox[data-side="right"]::after {
                        border-right: 10px solid #a00;
                    }
                    .tourguide-biglabel.tourguide-labelbox[data-side="up"]::after {
                        border-top: 10px solid #a00;
                    }
                    .tourguide-biglabel.tourguide-labelbox[data-side="down"]::after {
                        border-bottom: 10px solid #a00;
                    }
                    */
                    .tourguide-afterarea {
                        display: none;
                        flex-direction: row;
                        justify-content: space-around;
                    }
                    .tourguide-wrapper[data-islast="true"] .tourguide-afterarea {
                        display: flex;
                    }
                    .tourguide-autostartarea {
                        font-style: italic;
                        font-size: 90%;
                        padding: 0.5em 1em;
                    }
                    .tourguide-autostartarea .tourguide-dontshowtick {
                        vertical-align: middle;
                    }
                </style>
            `;
            return styles;
        };
    }

})(jQuery);
//}}}